DuelCountdown = {_G = _G}
setfenv(1, DuelCountdown)

local DuelCountdown = _G.DuelCountdown
local string, math, table = _G.string, _G.math, _G.table
local pairs, ipairs = _G.pairs, _G.ipairs
local assert, select, print = _G.assert, _G.select, _G.print

print = function(...)
  if debug then _G.print(...) end
end

-- Don't publish debugging versions; uncomment this line!
--debug = true

local MAGIC_NUMBER = 3

local handlerFrame = _G.CreateFrame("Frame")

-- Value of GetTime() at the start of the countdown timer animation.
local startTime

-- This function is definded in 'Timer.lua' and returns 'Horde', 'Alliance' or 'Neutral'.
-- Unfotunately it assumes we are in a rated battleground when we aren't in an arena and will return
-- 'PLAYER_FACTION_GROUP[GetBattlefieldArenaFaction()]' which happens to evaluate to 'Horde' when we
-- aren't actually in a rated battleground.
GetPlayerFactionGroup = _G.GetPlayerFactionGroup

function startTimer()
  -- TimerTracker is created from http://wowprogramming.com/utils/xmlbrowser/live/FrameXML/Timer.xml
  assert(_G.TimerTracker)

  -- This seeems to be the easiest way to stop all timers.
  _G.TimerTracker_OnEvent(_G.TimerTracker, "PLAYER_ENTERING_WORLD")

  -- I somewhat doubt we can do this without causing all kinds of taint issues. We'll see...
  _G.GetPlayerFactionGroup = function()
    -- With this return value, no texture will be shown as the timer expires; the last thing
    -- displayed it the big golden one.
    return "Neutral"
    -- This would be used if we wanted to display the horde or alliance texture:
    -- return _G.UnitFactionGroup("player")
  end
  _G.TimerTracker_OnEvent(_G.TimerTracker, "START_TIMER", _G.TIMER_TYPE_PVP, 3, 3)
  _G.GetPlayerFactionGroup = DuelCountdown.GetPlayerFactionGroup
  startTime = _G.GetTime()
end

function stopTimer()
  assert(_G.TimerTracker)
  _G.TimerTracker_OnEvent(_G.TimerTracker, "PLAYER_ENTERING_WORLD")
end

function handlerFrame:ADDON_LOADED(name)
  if name ~= "DuelCountdown" then return end
  self:UnregisterEvent("ADDON_LOADED")

  print("DuelCountdown loaded. Initializing...")

  handlerFrame:RegisterEvent("DUEL_REQUESTED")
  handlerFrame:RegisterEvent("DUEL_INBOUNDS")
  --handlerFrame:RegisterEvent("DUEL_OUTOFBOUNDS")
  handlerFrame:RegisterEvent("DUEL_FINISHED")
  handlerFrame:RegisterEvent("CHAT_MSG_SYSTEM")

  self.ADDON_LOADED = nil
end

function handlerFrame:DUEL_REQUESTED()
  print("DuelCountdown: handlerFrame:DUEL_REQUESTED called")
end

function handlerFrame:DUEL_INBOUNDS()
  print("DuelCountdown: handlerFrame:DUEL_INBOUNDS called")
end

function handlerFrame:DUEL_OUTOFBOUNDS()
  print("DuelCountdown: handlerFrame:DUEL_OUTOFBOUNDS called")
end

-- Since we are only interested in duels that were cancelled before they started (i.e. cancelled
-- while the countdown timer was running), maybe we could hook CancelDuel() or register one of
-- UI_ERROR_MESSAGE and UI_INFO_MESSAGE to check for the ERR_DUEL_CANCELLED ("Duel cancelled.")
-- message?
function handlerFrame:DUEL_FINISHED()
  print("DuelCountdown: handlerFrame:DUEL_FINISHED called")
  if _G.GetTime() - startTime < MAGIC_NUMBER then
    DuelCountdown.stopTimer()
  end
end

-- http://wowprogramming.com/docs/events/CHAT_MSG_SYSTEM
function handlerFrame:CHAT_MSG_SYSTEM(message, _, _, _, _, _, _, channelNumber)
  -- http://wowprogramming.com/utils/xmlbrowser/live/FrameXML/GlobalStrings.lua
  if string.find(message, _G.DUEL_COUNTDOWN) then
    if string.find(message, '3') then
      DuelCountdown.startTimer()
    end
  --[[
  elseif string.find(message, _G.DUEL_WINNER_KNOCKOUT) then
    -- ...
  elseif string.find(message, _G.DUEL_WINNER_RETREAT) then
    -- ...
  --]]
  end
end

handlerFrame:SetScript("OnEvent", function(self, event, ...)
  return self[event] and self[event](self, ...)
end)

--handlerFrame:SetScript("OnUpdate", function(self, elapsed)
--end)

handlerFrame:RegisterEvent("ADDON_LOADED")

-- Potentially relevant global strings we aren't currently using:
-- ERR_DUEL_CANCELLED, ...
-- http://wowprogramming.com/utils/xmlbrowser/live/FrameXML/GlobalStrings.lua
--
-- Some functions we could hook:
-- http://wowprogramming.com/docs/api_categories#duel
--
-- Other web links to pages that helped in the creation of this addon:
-- http://eu.battle.net/wow/en/forum/topic/2463327848
-- http://us.battle.net/wow/en/forum/topic/3871217857
-- http://wowprogramming.com/utils/xmlbrowser/live/FrameXML/Timer.lua
-- http://wowprogramming.com/docs/events/START_TIMER
-- http://dev.deadlybossmods.com/rss/DBM_Alpha_Changes-r10228.html
-- http://dev.deadlybossmods.com/rss/DBM_Alpha_Changes-r10230.html
-- http://wowprogramming.com/docs/api/UnitFactionGroup
-- http://wowprogramming.com/docs/events/CHAT_MSG_SYSTEM

-- vim:tw=100
